'use strict'
const path = require('path')
const fs = require('fs')
// agent.js 文件：
class AgentBoot {
  constructor(agent) {
    this.agent = agent
  }

  /**
   * Ready to call configDidLoad,
   * Config, plugin files are referred,
   * this is the last chance to modify the config.
   */
  configWillLoad() {
  }

  /**
   * Config, plugin files have been loaded.
   */
  configDidLoad() {
    /**
     * 是否开启agent模式
     */
    const basePath = path.join(this.agent.baseDir, 'app/bootstrap/agent')
    /**
     * 如果文件夹存在
     */
    if (fs.existsSync(basePath)) {
      fs.readdirSync(basePath, {
        withFileTypes: true
      }).forEach(dirent => {
        if (dirent.isFile() && path.extname(dirent.name) == '.js') {
          const file = path.join(basePath, dirent.name)
          const target = require(file)
          if (!(target instanceof Function)) {
            throw new Error(`${file} is not export a function.`)
          }
          target(this.agent)
        }
      })
    }
  }

  /**
   * All files have loaded, start plugin here.
   */
  async didLoad() {

  }

  /**
   * All plugins have started, can do some thing before agent ready
   */
  async willReady() {

  }

  /**
   * Worker is ready, can do some things
   * don't need to block the agent boot.
   */
  async didReady() {

  }

  /**
   * Server is listening.
   */
  async serverDidReady() {
  }

  /**
   * Do some thing before agent close.
   */
  async beforeClose() {
  }
}

module.exports = AgentBoot 